/* vlanTagLibP.h - private header file for Layer 2 VLAN tagging library */

/* Copyright 2002 - 2004 Wind River Systems, Inc. */

/*
modification history
--------------------
01h,07apr05,kch  Added addrFormat member to vlan_tag_data structure.
01g,15sep04,kc   Includes frame statistics only if debug is enabled.
01f,10sep04,kc   Added pVlanTagHdrCheck function pointer to vlan_tag_data
                 structure.
01e,19jun04,kc   Fixed SPR#98400 - added new ifIngressQtagAdmits and
                 ifEgressQtagXmits members to VLAN_TAG_DATA structure. Renamed
                 ifInputVidError to ifIngressDiscards and ifOutputPriorityError
                 to ifEgressPriorityError.
01d,08jun04,kc   Added updateFuncs boolean flag to VLAN_TAG_DATA structure.
01c,01jun04,kc   Added LLC_STP_LSAP define. Removed the if_tag_flags and
                 global_tag members from the VLAN_TAG_DATA structure. 
                 Added ifIngressType, ifIngressRules, ifEgressType and
                 ifEgressRules members to VLAN_TAG_DATA. Redefine function
                 pointer for tos2PriorityConvRn.
01b,28may04,kc   Added VLAN_TAG_DEFAULT_VID define to allow default vlan id
                 to be used for ingress/egress frames if the vlan id is not
                 explicitly configured.
01a,19may04,kc   written.
*/
 
#ifndef __INCvlantagLibPh
#define __INCvlantagLibPh

#ifdef __cplusplus
extern "C" {
#endif

/* macros */
#define IP_TOS_RESTORE(pMbuf)   ((pMbuf)->m_pkthdr.csum_flags >> 24) 

#define VLAN_TAG_CLEAR(pMbuf)   ((pMbuf)->m_pkthdr.csum_flags &= 0x0000ffff)

#define VLAN_TAG_SAVE(pMbuf,tag) ((pMbuf)->m_pkthdr.csum_flags |= ((tag) << 16))

#define VLAN_TAG_RESTORE(pMblk) ((pMblk)->m_pkthdr.csum_flags >> 16)


/* defines */ 

#define LLC_SNAP_FRAMELEN    8

#define NULL_VLAN_ID         0        /* for priority-tagged frame */
#define MIN_VLAN_ID          1        
#define MAX_VLAN_ID          4094
#define RESERVED_VLAN_ID     4095    

#define MIN_VLAN_PRIORITY    0
#define MAX_VLAN_PRIORITY    7

/* 
 * the following defines are used for manipulating the 4 byttes vlan Q-tag.
 * 
 */
#define VLAN_TAG_ID_MASK             0x0fff  /* 12 bits vlan id */ 
#define VLAN_TAG_THREE_BITS_MASK     0x0007  /* 3 bit priority field */
#define VLAN_TAG_PRIORITY_BITS_MASK  0xe000  /* bits 15..13 in the tag */ 
#define VLAN_TAG_PRIORITY_BITS_SHIFT 13      /* Bit shift for priority bits */
#define VLAN_TAG_CFI_BIT             0x1000  /* The CFI bit in the tag */  

/* system-wide default vlan id if one is not explicitly assigned */
#define VLAN_TAG_DEFAULT_VID         1       /* default vlan id */

/* LLC with DSAP and SSAP of 0x42 is reserved for IEEE 802.1 use. */
#define LLC_STP_LSAP                 0x42

#define ETHER_OTHER_FORMAT           0x00
#define ETHER_TYPE_2_FORMAT          0x01
#define ETHER_8023_FORMAT            0x02

/* debug facilities */
#undef VLAN_DEBUG

/* Structure definitions */  

/* 
 * an instance of vlan_tag_data structure will be added to each interface's
 * ifnet structure when the vlanTagIfAttach() is called. This data structure
 * contains data memebers needed for per-interface vlan manipulation. It is 
 * considered private and should only be used exclusively by the vlan tagging
 * library.
 */
typedef struct vlan_tag_data
    {
    ushort_t  ifIngressType;   /* acceptable ingress frame type */
    ushort_t  ifIngressRules;  /* ingress rules to apply */
    ushort_t  ifEgressType;    /* acceptable egress frame type */
    ushort_t  ifEgressRules;   /* egress rules to apply */
    ushort_t  ifVlanId;        /* VLAN id. lower 12 bits signifcant.*/
    uchar_t   ifPriority;      /* VLAN priority bits, only 3 bits significant */
    uchar_t   addrFormat;      /* (ethernet) addressing format */
    BOOL      updateFuncs;     /* if set, update end driver functions table */

    TOS2_PRIORITY_FUNCPTR   tos2PriorityConvRtn;  /* function pointer */    

    /* 
     * function pointer to validate received frame's vlan tag header. This
     * allow each interface to have its own unique classification rules
     * which may not be too bad after all...
     */
    STATUS (*pVlanTagHdrCheck) (M_BLK_ID pMblk, struct ifnet * pIfp);

    /* per interface VLAN statistics for debugging and diagnostic purposes */
#ifdef VLAN_DEBUG
    /*
     * the number of transmitted priority-tagged frames discarded due to
     * priority-bits errors
     */
    uint32_t  ifEgressPriorityError;

    /*
     * the number of valid Q-Tag frames transmitted by this interface to
     * its segment for this VLAN
     */
    uint32_t  ifEgressQtagXmits;

    /*
     * the number of frames received by this interface which were discarded
     * due to VLAN related reasons (such as vlan id mismatched or the
     * received frane is an untagged or priority-tagged frame but the
     * ingress acceptable frame type is configured to received tagged
     * frame only
     */
    uint32_t  ifIngressDiscards;

    /*
     * the number of valid frames received by this interface from its
     * segment which were classified as belonging to this VLAN. Note
     * that a frame received on this interface is counted if and only
     * if it will be forwarded to upper layer for further processing.
     */
    uint32_t  ifIngressQtagAdmits;
#endif /* VLAN_DEBUG */    
    } VLAN_TAG_DATA;  

#ifdef __cplusplus
}
#endif

#endif /* __INCvlantagLibPh */

